@@ -5,4 +5,6 @@ from .t5.t5_tokenizer import T5BPETokenizer
 from .roberta.roberta_tokenizer import ROBERTATokenizer
 from .bert.bert_tokenizer import BertWordPieceTokenizer
 from .cpm_1.cpm1_tokenizer import CPMTokenizer
-from .opt.opt_en_tokenizer import OPTTokenizer
\ No newline at end of file
+from .opt.opt_en_tokenizer import OPTTokenizer
+from .uni_tokenizer.tokenizer import Tokenizer
+# from .uni_tokenizer.base_tokenizer import BaseTokenizer
